/*****************************************************************************
 * 
 * PROJECT: Task Control Management.
 *
 * (c) Copyright 1996 Reid Simmons.  All rights reserved.
 *
 * FILE: tcmGlobal.cc
 *
 * ABSTRACT: (Internal) global variables
 *
 * $Source: /afs/cs/project/TCA/Master/tcaV8/tcm/src/tcmGlobal.cc,v $ 
 * $Revision: 1.3 $
 * $Date: 1999/06/06 13:48:09 $
 * $Author: reids $
 *
 * REVISION HISTORY:
 *
 * $Log: tcmGlobal.cc,v $
 * Revision 1.3  1999/06/06 13:48:09  reids
 * Changes to TCM 1.3.0:
 *   New list data structures.
 *   New, generalized logging interface.
 *   Fixed firstChild to work if there are no children.
 *   Fixed code to deallocate a node.
 *   In certain circmumstances, don't send termination signal when task
 *     itself terminating.
 *
// Revision 1.2  97/12/04  17:50:25  reids
// Another fairly stable version (except that monitors do not quite work)
// 
// Revision 1.1  97/11/21  14:06:43  reids
// First release of TCM -- seems to be a stable version
// 
 *
 *****************************************************************************/

#include "tcmGlobal.h"

Global_State globalState;

static Task_Tree_Node globalRoot("Root Node");

void initializeGlobalState(void)
{
  GET_TCM_GLOBAL(terminalLogger).setLogging(stderr, FALSE);

  // Need to artificially increment the refcount, or else the static
  // globalRoot gets freed when the program exits!
  globalRoot.increment();
  GET_TCM_GLOBAL(rootNode) = &globalRoot;

  GET_TCM_GLOBAL(rootNode)->transitionTo(Planning_State);
  GET_TCM_GLOBAL(rootNode)->transitionTo(Achieving_State);
}
