/* Agent A (port 1381) defines tasks A, C and E;
   Agent B (port 1382) defines tasks B and D.
*/ 

#include <stdio.h>
#include <stdlib.h>
#include "tcm.h"
#include "ipc.h"

#include "distrTests.h"

#define THIS_AGENT  AGENT_B
#define OTHER_AGENT AGENT_A

#define THIS_HOST   ":" CENTRAL_HOST_B
#define OTHER_HOST  ":" CENTRAL_HOST_A

int whichTest = BASIC_TEST;

// Forward references
static void BHnd (const TCM_Task_Tree_Ref &node, const void *data);
static void DHnd (const TCM_Task_Tree_Ref &node, const void *data);

static void printMsg (const void *data)
{ 
#ifdef PRINT_OUT
  fprintf(stderr, "INVOCATION: %s\n", (const char *)data);
#else
  UNUSED(data);
#endif
}

static void BHnd (const TCM_Task_Tree_Ref &node, const void *data)
{
#ifdef PRINT_OUT
  fprintf(stderr, "Invoking BHnd %d\n", (int)data);
#else
  UNUSED(data);
#endif

  if (DOING_TEST(DEALLOCATE_TEST1)) {
    TCM_Task_Tree_Ref E1 = TCM_CreateDistributedNode(OTHER_AGENT, node, "E1",
						     NULL);
    TCM_Task_Tree_Ref E2 = TCM_AllocateDistributedNode(OTHER_AGENT, "E2");
    // NOTE: Do E2 *before* E1 !!!
    TCM_DelayUntil(TCM_StartOf(TCM_AchievingOf(E1)), 
		   TCM_EndOf(TCM_AchievingOf(E2)));
    TCM_DeallocateNode(E2);

    TCM_Success(node);
    return;
  }

  if (DOING_TEST(TERMINATE_TEST5) || 
      DOING_TEST(TERMINATE_AT_TEST2) || DOING_TEST(TERMINATE_AT_AFTER_TEST3))
    TCM_TerminateNode(node);

  TCM_Task_Tree_Ref E1 = TCM_CreateDistributedNode(OTHER_AGENT, node, "E1",
						   NULL);
  TCM_Task_Tree_Ref E2 = TCM_CreateDistributedNode(OTHER_AGENT, node, "E2",
						   NULL);
  if (DOING_TEST(TERMINATE_TEST16)) TCM_TerminateNode(E2);
  if (DOING_TEST(TERMINATE_TEST17)) TCM_TerminateNode(E1);
  // NOTE: Do E2 *before* E1 !!!
  TCM_DelayUntil(TCM_StartOf(TCM_AchievingOf(E1)),
		 TCM_EndOf(TCM_AchievingOf(E2)));

  if (DOING_TEST(TERMINATE_TEST15)) TCM_TerminateNode(E2);
  if (DOING_TEST(SIGNAL_TEST2)) {
    TCM_InvokeWhen(TCM_EndOf(TCM_AchievingOf(E1)),
		   printMsg, "End Achieving E1");
    TCM_InvokeWhen(TCM_StartOf(TCM_AchievingOf(E2)),
		   printMsg, "Start Achieving E2");
    TCM_InvokeWhen(TCM_StartOf(TCM_PlanningOf(node)),
		   printMsg, "Start Planning B");
    TCM_InvokeWhen(TCM_EndOf(TCM_PlanningOf(node)),
		   printMsg, "End Planning B");
  }

  TCM_Success(node);

  if (DOING_TEST(TERMINATE_TEST6) || DOING_TEST(TERMINATE_ON_TERMINATION1))
    TCM_TerminateNode(node);
}

static void DHnd (const TCM_Task_Tree_Ref &node, const void *data)
{
#ifdef PRINT_OUT
  fprintf(stderr, "Invoking DHnd %d\n", (int)data);
#else
  UNUSED(data);
#endif
  if (DOING_TEST(TERMINATE_TEST9)) TCM_TerminateNode(node);

  if (DOING_TEST(TERMINATE_AT_AFTER_TEST1) ||
      DOING_TEST(TERMINATE_AT_AFTER_TEST2) ||
      DOING_TEST(TERMINATE_AT_AFTER_TEST3))
    TCM_WaitFor(seconds(10));

  TCM_Success(node);

  if (DOING_TEST(TERMINATE_TEST10)) TCM_TerminateNode(node);
}

/****************************************************************
 *  DISTRIBUTED ALLOCATION AND ACTION FUNCTIONS
 ****************************************************************/

class B_Action : public _Action
{
 public: 
  B_Action ( const TCM_Task_Tree_Ref & task, void *args) {}

  virtual void execute ( const TCM_Task_Tree_Ref & task ) { BHnd(task, NULL); }
};

static TCM_Task_Tree_Ref allocateB (void)
{
  return TCM_AllocateGoalNode("B");
}

static TCM_Action_Ref actionB (const TCM_Task_Tree_Ref &B, void *args)
{
  return new B_Action(B, args);
}

class D_Action : public _Action
{
 public: 
  D_Action ( const TCM_Task_Tree_Ref & task, void *args) {}

  virtual void execute ( const TCM_Task_Tree_Ref & task ) { DHnd(task, NULL); }
};

static TCM_Task_Tree_Ref allocateD (void)
{
  return TCM_AllocateGoalNode("D");
}

static TCM_Action_Ref actionD (const TCM_Task_Tree_Ref &D, void *args)
{
  return new D_Action(D, args);
}

/****************************************************************/

int main (int argc, char **argv)
{
  if (argc > 1) {
    whichTest = atoi(argv[1]);
  }

  TCM_Initialize();

#ifdef PRINT_OUT
  TCM_SetTerminalLoggingOptions(Log_Msg | Log_Status | Log_Id | Log_ParentId);
  // TCM_SetTerminalLoggingOptions(Log_Time | TCM_TerminalLoggingOptions());
#else
  TCM_SetTerminalLoggingOptions(0);
#endif
  
  //IPC_setVerbosity(IPC_Silent);
  TCM_EnableDistributedComm(THIS_AGENT, THIS_HOST);
  TCM_ConnectDistributedAgent(OTHER_AGENT, OTHER_HOST);

  TCM_RegisterDistributedTask("B", allocateB, actionB, NULL);
  TCM_RegisterDistributedTask("D", allocateD, actionD, NULL);
  if (DOING_TEST(EXCEPTION_TEST1) || DOING_TEST(EXCEPTION_TEST2)) {
    TCM_RegisterDistributedException(EXCEPTION_NAME, EXCEPTION_FORMAT);
  }

  TCM_ProcessAgenda();

  return 0;
}
