
import netscape.application.Color;

public class DataConstraintStatement extends DataStatement
				  implements DataConstrainedObject
{
	/* Class variables */
  public final static String   BEGIN_TAG_TASK_INDEX = "[";
  public final static String   END_TAG_TASK_INDEX   = "]";
  public final static String   TASK_TAG             = "T";
  public final static String   SEMICOLON            = ";";

  public       static Color    CONSTRAINT_COLOR     = Color.red;


	/* Instance Variables */
  protected String           taskTag;
  protected String[]         taskTagIndexes;
  protected DataConstraint   constraint;

	/* Instance Methods */
  public DataConstraintStatement ( )
  {
    taskTag        = null;
    taskTagIndexes = null;
    constraint     = new DataConstraint();
    constraint . setParent ( this );
  }

  public boolean getHasTaskTag()
	      { return DataComponent.isEmptyString ( getTaskTag() ) == false; }

  public String getTaskTag ( )    { return taskTag == null ? "" : taskTag; }
  public void   setTaskTag ( String theTaskTag )    { taskTag = theTaskTag; }


  public boolean getHasTaskIndexes ( )
  {
    return (   ( getTaskIndexes()          != null )
	    && ( getTaskIndexes() . length >  0    ) );
  }

  public String[] getTaskIndexes () { return taskTagIndexes; }

  public void     setTaskIndexesWithoutParsing ( String[] theTaskIndexes )
  {
    taskTagIndexes = theTaskIndexes;
  }


  public DataConstraint  getConstraint()  { return constraint; }


	/* DataConstrainedObject Interface */
  public int getConstraintCount() { return 1; }
  public DataConstraint  getConstraint( int theIndex )
  {
    if ( theIndex == 0 )
    {
      return getConstraint();
    }
    else
    {
      System.err.println ( "[DataConstraintStatement:getConstraint]  Error:  "
			   + "Invalid Index (" + theIndex + ")" );
      return null;
    }
  }
  public void addConstraint ( DataConstraint theConstraint )
  {
    if ( theConstraint != null )
    {
      constraint = theConstraint;
      theConstraint . setParent ( this );
    }
    else
    {
      System.err.println ( "[DataConstraintStatement:addConstraint]  Error:  "
			   + "NULL Constraint." );
    }
  }



	/* Allow searching for contained expression subparts... */
  public boolean hasSubcomponentFraction ( String theString )
  {
    return super               . hasSubcomponentFraction ( theString )
      ||   getConstraint ( 0 ) . hasSubcomponentFraction ( theString );
  }


  public DataComponent newComponent ( )
  {
    DataConstraintStatement   constraintStatement
					 = new DataConstraintStatement();
    constraintStatement . addSubcomponent ( "\n" );
    constraintStatement . setIndex ( DataConstraintStatement.TASK_TAG );
    constraintStatement . setIndex ( DataConstraintStatement.SEMICOLON );

    return constraintStatement;
  }

  	/* Validates code that is inside of a Task */
  public void validateInternalCode( int                         theReference,
				    DataValidateCodeReturnValue theReturnValue)
    throws CompilationException
  {
    DataComponent       ancestor, taskTagDataComponent;
    DataTaskDefinition  ancestorDataTaskDefinition;
    int                 iterationCount;

    if ( getHasTaskTag() )
    {
    	/* Find our DataTaskDefinition Ancestor */
      for ( ancestor  = this;
	    ancestor != null;
	    ancestor  = ancestor . getParent()
	   )
      {
	if ( ancestor instanceof DataTaskDefinition )
	  break;
      }
      if ( ancestor == null )
      {
	throw new CompilationException (
		  "Line:" + getLineNumberString() + ": Programmer Error:  "
		  + "Unable to find DataTaskDefinition ancestor." );
      }
      ancestorDataTaskDefinition = (DataTaskDefinition) ancestor;

	/* Find the taskTagDataComponent */
      taskTagDataComponent = ancestorDataTaskDefinition
			      . getDataComponentWithName ( getTaskTag() );
	/* If this is null, the error will be picked up elsewhere. */
      if ( taskTagDataComponent != null )
      {
	  /* Find out how many iteration loops enclose the tagTask Object */
	  /* Note:  This can throw a CompilationException.                */
	iterationCount = ancestorDataTaskDefinition
			   . getIterationParentCount ( taskTagDataComponent );


	  /* And lets double-check this stuff. */
	if (   ( getHasTaskIndexes()       == true           )
	    && ( getTaskIndexes() . length >  iterationCount ) )
	{
	  theReturnValue
	    . addError ( getLineNumberString() )
	    . write ( "Too many Iteration-Indexes for Constraint "        )
	    . write ( " Statement Reference.  (Reference is enclosed in " )
	    . write ( "" + iterationCount                                 )
	    . write ( " iteration loop(s) and there is (are) "            )
	    . write ( "" + getTaskIndexes() . length                      )
	    . write ( " array index(es) specified.)\n"                    );
	}

	if (   (     iterationCount            >  0                )
	    && (   ( getTaskIndexes()          == null           )
		|| ( getTaskIndexes() . length <  iterationCount ) ) )
	{
	  theReturnValue
	    . addWarning ( getLineNumberString() )
	    . write ( "Using an Iteration-Set of References for Constraint" )
	    . write ( " Statement Reference.  (Reference is enclosed in "   )
	    . write ( "" + iterationCount                                   )
	    . write ( " iteration loop(s) and there is (are) "              )
	    . write (   ( getTaskIndexes() == null )
		      ? "0" : ("" + getTaskIndexes() . length)              )
	    . write ( " array index(es) specified.)\n"                      );
	}
      } /* if ( taskTagDataComponent != null ) */
    } /* if ( getHasTaskTag() ) */



    for ( int i=0;
	  getHasTaskIndexes() && (i < getTaskIndexes().length);
	  i++ )
    {
      if (   ( getTaskIndexes()[i] . trim() . charAt ( 0 ) == '0' )
	  && ( getTaskIndexes()[i] . trim() . length()     >   1  ) )
      {
	theReturnValue
	  . addWarning ( getLineNumberString() )
	  . write ( "Task Index is specified in octal.\n" );
	break; /* Don't test any more indexes... */
      }
    }

	/* Perform super-class validation */
    super . validateInternalCode ( theReference, theReturnValue );

    if ( getConstraint() . getConstraintType() == DataConstraint.PARALLEL )
    {
      theReturnValue
	. addWarning ( getConstraint() . getLineNumberString() )
	. write ( "PARALLEL constraint in a CONSTRAINT STATEMENT is "
		  + "meaningless.\n" );
    }

    getConstraint() . validateInternalCode ( theReference, theReturnValue );
  }


  protected
    DataComponent parseSpecificComponent ( TDLParser theTDLParser,
					   int       theObjectSubsetToParse )
    throws Exception
  {
    warnIfInvalidObjectSubset ( theObjectSubsetToParse,
				"parseSpecificComponent", false );

    return theTDLParser . parseConstraintStatement ( 0 );
  }


  public boolean setFrom ( DataComponent  theObjectToCopy,
			   int            theObjectSubsetToSet,
			   boolean        theCanDestroyObject  )
  {
    if ( /*NOT*/ !  ( theObjectToCopy instanceof DataConstraintStatement ) )
    {
      System.err.println ( "[DataConstraintStatement:setFrom]  Error:  "
			   + "Illegal object.  (\""
			   + theObjectToCopy . getClass() . getName()
			   + "\")" );
      return false;
    }

    DataConstraintStatement   theConstraintStatementToCopy
				   = (DataConstraintStatement) theObjectToCopy;


    setTaskTag ( theConstraintStatementToCopy . getTaskTag() );

    if (    theCanDestroyObject 
	 || theConstraintStatementToCopy.taskTagIndexes == null )
    {
      taskTagIndexes = theConstraintStatementToCopy.taskTagIndexes;
    }
    else
    {
      taskTagIndexes
	= new String [ theConstraintStatementToCopy.taskTagIndexes.length ];
      System.arraycopy ( theConstraintStatementToCopy.taskTagIndexes, 0,
			 taskTagIndexes, 0,
			 taskTagIndexes.length );
    }


    if ( getConstraint()
	   . setFrom ( theConstraintStatementToCopy . getConstraint(),
		       theObjectSubsetToSet, theCanDestroyObject )
	 == false )
    {
      return false;
    }

    return super . setFrom ( theObjectToCopy, theObjectSubsetToSet,
			     theCanDestroyObject );
  }


  protected void generateTaskTag ( DataDestination  theOutputDestination,
				   int              theObjectSubsetToGenerate )
    throws CompilationException
  {
    String[] taskIndexes = getTaskIndexes();

	/* Idiocy check */
    if ( getHasTaskTag() == false )
    {
      throw new CompilationException ( "Line:" + getLineNumberString()
		      + ":  [DataConstraintStatement:generateEventTagTask]  "
		      + "Error:  No Task-Tag available to generate..." );
    }

	/* Write our task-tag */
    theOutputDestination . write ( getTaskTag() );


	/* If we do have indexes... */
    if ( getHasTaskIndexes() )
    {
      for ( int i=0;   i < taskIndexes.length;   i++ )
      {
	  /* Write non-significant tokens & open-index */
	generateSubcomponents (
			     DataConstraintStatement.BEGIN_TAG_TASK_INDEX + i,
			     theOutputDestination,
			     theObjectSubsetToGenerate, false );
	theOutputDestination
	  . write ( DataConstraintStatement.BEGIN_TAG_TASK_INDEX );


	  /* Write non-significant tokens & index */      
	generateSubcomponents ( "" + i,
				theOutputDestination,
				theObjectSubsetToGenerate, false );
	if ( taskIndexes [ i ] != null )
	{
	  theOutputDestination . write ( taskIndexes [ i ] );
	}

	  /* Write non-significant tokens & close-index */
	generateSubcomponents ( DataConstraintStatement.END_TAG_TASK_INDEX + i,
				theOutputDestination,
				theObjectSubsetToGenerate, false );
	theOutputDestination . write ( DataConstraint.END_TAG_TASK_INDEX );
      }
    }
  }




  public void generate ( DataDestination  theOutputDestination,
			 int              theObjectSubsetToGenerate )
  {
       	/* Print a warning if necessary... */
    warnIfInvalidObjectSubset ( theObjectSubsetToGenerate, "generate", true );

        	/* Initialize us to generate non-significant tokens... */
    initializeGenerateSubcomponentIndex();

	/* Generate any labels that we have... */
    generateLabels ( theOutputDestination, theObjectSubsetToGenerate,
		     DataConstraintStatement.TASK_TAG );

	/* Write any pre-task-tag non-significant tokens */
    generateSubcomponents ( DataConstraintStatement.TASK_TAG,
			    theOutputDestination,
			    theObjectSubsetToGenerate, false );

	/* If we are doing CXX generation... */
    if ( isCxxSubset ( theObjectSubsetToGenerate ) )
    {
      theOutputDestination . setNewlineText ( "    // " );
      theOutputDestination . write          ( "    // " );
    }

	/* Write our task-tag */
    generateTaskTag ( theOutputDestination, DataComponent.ENTIRE_OBJECT );


	/* Write our constraint */
    getConstraint() . generate ( theOutputDestination,
				 DataComponent.ENTIRE_OBJECT );


	/* Write any pre-";" non-significant tokens */
    generateSubcomponents ( DataConstraintStatement.SEMICOLON,
			    theOutputDestination,
			    theObjectSubsetToGenerate, false );

	/* Write ";" */
    theOutputDestination . write ( DataSpawnTask.SEMICOLON );


	/* Write any remaining non-significant tokens */
	/* (There *SHOULD* *NOT* be any...  But just in case...) */
    generateAllRemainingSubcomponents ( theOutputDestination,
					theObjectSubsetToGenerate, false );

	/* If we are doing CXX generation... */
    if ( isCxxSubset ( theObjectSubsetToGenerate ) )
    {
      theOutputDestination . clearNewlineText();
      theOutputDestination . write ( "\n" );
      generateCxx ( theOutputDestination, theObjectSubsetToGenerate );
      return;
    }
  }


  public void generateCxx ( DataDestination  theOutputDestination,
			    int              theSubsetToProduce   )
    throws CompilationException
  {
    int  indent;

    if ( getHasTaskTag() == false )
    {
      throw new CompilationException ( "Line:" + getLineNumberString()
				       + ":  Error:  Missing Task-Tag." );
    }

	/* Deal with #line macros */
    theOutputDestination . setUsingTdlFileName ( false );
    theOutputDestination . write ( "\n" ); /* This newline will be skipped.*/


	/* Write "_TDL_SpawnedTasks . applyConstraintTo ( " */
    theOutputDestination
      . write ( DataComponent.CXX_TDL_TASK_HANDLER_INSTANCE );
    theOutputDestination . write ( " . " );
    theOutputDestination
      . write ( DataComponent.CXX_TDL_TASK_HANDLER_APPLY_CONSTRAINT_TO );
    theOutputDestination . write ( " ( " );

    indent = theOutputDestination . indentToCurrentColumn();


	/* Write the corresponding CXX generated string. */
    if ( getTaskTag() . equals ( DataComponent.THIS ) )
      theOutputDestination
	. write ( DataComponent.CXX_ENCLOSING_TASK_CONSTRAINT_OBJECT );
    else
      theOutputDestination
	. write ( DataStatement.getTagTaskCxxReferenceString( getTaskTag(),
							      getTaskIndexes(),
							      this ) );
    theOutputDestination . write ( ",\n" );



	/* Generate constraint in the inside-task style. */
    getConstraint() . generateCxxTaskInternal ( theOutputDestination );

    theOutputDestination . removeIndent ( indent );

    theOutputDestination . write ( " );\n" );
  }




  public boolean isSpawnRelatedStatement ( )
  {
    return true;
  }

  public int  getChildStatementCount()
  {
    return super . getChildStatementCount()
      + getConstraintChildStatementCount();
  }

  public DataStatement  getChildStatement ( int theIndex )
  {
    if ( theIndex < getConstraintChildStatementCount() )
      return getConstraintChildStatement ( theIndex );
    else
      return super . getChildStatement(   theIndex
					- getConstraintChildStatementCount() );
  }



	/********************************************************/
        /********** Methods to interface with Graphing **********/
	/********************************************************/
  public boolean supportsGraphType ( int theGraphType )
  {
    if (   ( theGraphType == GraphRoot.GUI_TREE_GRAPH ) )
      return true;
    else
      return false;
  }

  public GraphModule createGraphModule ( int theGraphType )
  {
    if (   ( theGraphType != GraphRoot.GUI_TREE_GRAPH ) )
      return null;

    /* else ... */

    /* Building:  +- "[.]"
     *            |
     */

    ScrollTextOfDataComponent  text = new ScrollTextOfDataComponent ( this );

    GraphNode       node = new GraphNode ( text,
					   GraphComponent.LEFT,
					   DataJumpStatement.JUMP_COLOR,
					   true, null );
						     
    GraphConnection toNode = new GraphConnection( GraphComponent.LEFT,
						  DataJumpStatement.JUMP_COLOR,
						  false, false, node );

    GraphMeet       startMeet = new GraphMeet ( GraphComponent.UP,
						DataJumpStatement.JUMP_COLOR,
						GraphComponent.LEFT,
						toNode );

    GraphConnection tail = new GraphConnection ( GraphComponent.UP,
					     DataComponent.getDraggableColor(),
					     true, true, null );
    startMeet . addChild ( tail );

    return new GraphModule ( startMeet, tail, null, null,
			     this, true, theGraphType );
  }

	/* Invoked by this.update(*) invoking GraphModule.update() */
  public boolean updateGraphModule ( DataComponent  theModifiedComponent,
				     Object         theStoredData,
				     GraphModule    theGraphModule )
  {
	/* There is nothing to do here.. */
    return true;
  }

}

