
import netscape.application.Color;

/*
 * This represents an arbitrary C/C++ Expression Statement
 */

public class DataExpressionStatement extends DataStatement
{
	/* Class Constants. */
  public final static String   EXPRESSION_INDEX = "expressionIndex";
  public final static String   SEMICOLON        = ";";

  public final static int      PARSE_GUI_EXPRESSION_STATEMENT = 5001;


	/* Class Variables */
  public       static Color    EXPRESSION_COLOR = Color.pink;


	/* Instance Variables */
  protected DataExpression   expression;


	/* Instance Methods */
  public DataExpressionStatement()
  {
    expression = null;
  }

  public DataExpression getExpression () { return expression; }

  public void           setExpression ( DataExpression theExpression )
  {
    if ( expression != null )
      expression . setParent ( null );

    expression = theExpression;

    if ( expression != null )
      expression . setParent ( this );
  }


	/* Allow searching of our expression... */
  public boolean hasSubcomponentFraction ( String theString )
  {
    return super               . hasSubcomponentFraction ( theString )
      || (   ( getExpression() != null )
	  && ( getExpression() . hasSubcomponentFraction ( theString ) ) );
  }


  public DataComponent newComponent ( )
  {
    DataExpressionStatement  statement = new DataExpressionStatement();

    statement . addSubcomponent ( "\n" );
    statement . setIndex ( DataExpressionStatement.EXPRESSION_INDEX );
    statement . setIndex ( DataExpressionStatement.SEMICOLON        );

    return statement;
  }

  public String getWarnString ( int      theObjectSubset,
				boolean  theIsGenerating )
  {
    return super . getWarnString ( theObjectSubset, theIsGenerating )
	      + " or  DataExpressionStatement.PARSE_GUI_EXPRESSION_STATEMENT ("
	      + DataExpressionStatement.PARSE_GUI_EXPRESSION_STATEMENT + ")";
  }

  public boolean isValidObjectSubset ( int      theObjectSubset,
				       boolean  theIsGenerating )
  {
    if ( theObjectSubset
	   == DataExpressionStatement.PARSE_GUI_EXPRESSION_STATEMENT )
      return true;
    else
      return super . isValidObjectSubset ( theObjectSubset, theIsGenerating );
  }


  protected
    DataComponent parseSpecificComponent ( TDLParser theTDLParser,
					   int       theObjectSubsetToParse )
    throws Exception
  {
    warnIfInvalidObjectSubset ( theObjectSubsetToParse,
				"parseSpecificComponent", false );
    return theTDLParser . parseExpressionStatement ( 0 );
  }


  public boolean setFrom ( DataComponent  theObjectToCopy,
			   int            theObjectSubsetToSet,
			   boolean        theCanDestroyObject  )
  {
    if ( /*NOT*/ !  ( theObjectToCopy instanceof DataExpressionStatement ) )
    {
      System.err.println ( "[DataExpressionStatement:setFrom]  Error:  "
			   + "Illegal object.  (\""
			   + theObjectToCopy . getClass() . getName()
			   + "\")" );
      return false;
    }

    DataExpressionStatement  theExpressionStatementToCopy
			        = (DataExpressionStatement) theObjectToCopy;
    DataComponent            aObject
                                = theExpressionStatementToCopy.expression;
    boolean                  returnValue;


    if (    ( theCanDestroyObject == false )
	&&  ( aObject             != null  ) )
    {
      aObject =  aObject . copyComponent();
     
      if ( aObject == null )
      {
	System.err.println ( "[DataExpressionStatement:setFrom]  ERROR:  "
			     + "Unable to replicate object:  "
			     +  theExpressionStatementToCopy.expression );
	return false;
      }
    }

    setExpression ( (DataExpression) aObject );


    returnValue =  super . setFrom ( theObjectToCopy, theObjectSubsetToSet,
				     theCanDestroyObject );


	/* Kludge to add "\n"'s to GUI-parsed jump-statements */
    if (   ( returnValue                                           == true )
	&& ( theObjectSubsetToSet
	        == DataExpressionStatement.PARSE_GUI_EXPRESSION_STATEMENT  )
	&& ( getIndex ( DataExpressionStatement.EXPRESSION_INDEX ) == 0    ) )
    {
      addSubcomponent ( "\n", 0 );

      setIndex (            DataExpressionStatement.EXPRESSION_INDEX,
		 getIndex ( DataExpressionStatement.EXPRESSION_INDEX ) + 1 );

      setIndex (            DataExpressionStatement.SEMICOLON,
		 getIndex ( DataExpressionStatement.SEMICOLON ) + 1 );
    }

    return returnValue;
  }


  public void generate ( DataDestination  theOutputDestination,
			 int              theObjectSubsetToGenerate )
  {
	/* Hack to deal with GUI-parsing... */
    if ( theObjectSubsetToGenerate
	   == DataExpressionStatement.PARSE_GUI_EXPRESSION_STATEMENT )
      theObjectSubsetToGenerate = DataComponent . ENTIRE_OBJECT;

       	/* Print a warning if necessary... */
    warnIfInvalidObjectSubset ( theObjectSubsetToGenerate, "generate", true );

    	/* Initialize us to generate non-significant tokens... */
    initializeGenerateSubcomponentIndex();

	/* Generate any labels that we have... */
    generateLabels ( theOutputDestination, theObjectSubsetToGenerate,
		     DataExpressionStatement.EXPRESSION_INDEX,
		     getExpression(), DataExpression.FIRST_TOKEN_INDEX );

	/* Write any pre-expression non-significant tokens */
	/* (just in case... There *should* not be any...)  */
    generateSubcomponents ( DataExpressionStatement.EXPRESSION_INDEX,
			    theOutputDestination,
			    theObjectSubsetToGenerate, false );

	/* Write our expression */
    if ( getExpression() != null )
      getExpression() . generate ( theOutputDestination,
				   theObjectSubsetToGenerate );

	/* Write any non-significant tokens before the semicolon... */
    generateSubcomponents ( DataExpressionStatement.SEMICOLON,
			    theOutputDestination,
			    theObjectSubsetToGenerate, false );

	/* Write our semicolon */
    theOutputDestination . write ( DataExpressionStatement.SEMICOLON );

	/* Write any remaining non-significant tokens */
	/* (There *SHOULD* *NOT* be any...  But just in case...) */
    generateAllRemainingSubcomponents ( theOutputDestination,
					theObjectSubsetToGenerate, false );
  }



	/********************************************************/
        /********** Methods to interface with Graphing **********/
	/********************************************************/
  public boolean supportsGraphType ( int theGraphType )
  {
    if (   ( theGraphType == GraphRoot.GUI_TREE_GRAPH ) )
      return true;
    else
      return false;
  }

  public GraphModule createGraphModule ( int theGraphType )
  {
    if (   ( theGraphType != GraphRoot.GUI_TREE_GRAPH ) )
      return null;

    /* else ... */

    /* Building:  +- "[.]"
     *            |
     */

    ScrollTextOfDataComponent  expressionText
				 = new ScrollTextOfDataComponent ( this,
		      DataExpressionStatement.PARSE_GUI_EXPRESSION_STATEMENT );

    GraphNode       expressionNode = new GraphNode ( expressionText,
						     GraphComponent.LEFT,
						     DataExpressionStatement
						      . EXPRESSION_COLOR,
						     true, null );
						     
    GraphConnection toExpression = new GraphConnection( GraphComponent.LEFT,
							DataExpressionStatement
							 . EXPRESSION_COLOR,
							false, false,
							expressionNode );

    GraphMeet       startMeet = new GraphMeet ( GraphComponent.UP,
						DataExpressionStatement
						 . EXPRESSION_COLOR,
						GraphComponent.LEFT,
						toExpression );

    GraphConnection tail = new GraphConnection ( GraphComponent.UP,
					     DataComponent.getDraggableColor(),
					     true, true, null );
    startMeet . addChild ( tail );

    return new GraphModule ( startMeet, tail, null, null,
			     this, true, theGraphType );
  }


	/* Invoked by this.update(*) invoking GraphModule.update() */
  public boolean updateGraphModule ( DataComponent  theModifiedComponent,
				     Object         theStoredData,
				     GraphModule    theGraphModule )
  {
	/* There is nothing to do here.. */
    return true;
  }

}
