
import netscape.application.*;
import netscape.util.*;
import netscape.constructor.*;
import java.io.*;


public class SearchDirectoryWindowHandler
			         extends WindowHandlerAbstractBaseClass
		              implements ExtendedTarget, Target,
					 TextFieldOwner, TextViewOwner,
					 java.lang.Runnable
{
	/* Constant (Special) Filenames */
  public final static String SEARCH_DIRECTORY_PLAN_NAME
					       = "SearchDirectoryWindow.planb";

	/* SEARCH_DIRECTORY_PLAN_NAME Objects */
  public final static String CASE_SENSITIVE_BUTTON_NAME= "CaseSensitiveButton";
  public final static String SUBDIRECTORIES_BUTTON_NAME
						= "SearchSubdirectoriesButton";
  public final static String SEARCH_BUTTON_NAME        = "SearchButton";
  public final static String STOP_BUTTON_NAME          = "StopButton";
  public final static String DIRECTORY_TEXTFIELD_NAME  = "DirectoryTextField";
  public final static String SEARCH_STRING_TEXTFIELD_NAME
						     = "SearchStringTextField";
  public final static String SEARCHING_TEXTFIELD_NAME  = "SearchingTextField";
  public final static String FOUND_SCROLLGROUP_NAME    = "FoundScrollGroup";


	/* Commands */
  public final static String SELECT_DIRECTORY_COMMAND  = "SelectDirectory";
  public final static String SEARCH_DIRECTORY_COMMAND  = "SearchDirectory";
  public final static String STOP_SEARCH_COMMAND       = "StopSearch";

	/* Private Commands */
  private final static String SEARCHING_FILE     = "SearchingFile";
  private final static String FOUND_FILE         = "FoundFile";
  private final static String FINISHED_SEARCHING = "FinishedSearching";

	/* Constants */
  public final static String TITLE                 = "Search Directory";



	/* Class variables */
  protected static Plan      searchDirectoryWindowPlan = null;

	/* Class Methods */
  protected static synchronized Plan getSearchDirectoryWindowPlan()
  {
    if ( SearchDirectoryWindowHandler.searchDirectoryWindowPlan == null )
    {
      SearchDirectoryWindowHandler.searchDirectoryWindowPlan
	= WindowHandlerAbstractBaseClass.getPlan (
	    SearchDirectoryWindowHandler.SEARCH_DIRECTORY_PLAN_NAME,
	    "[SearchDirectoryWindowHandler:getSearchDirectoryWindowPlan]  "
	    + "Error:  Failure - Unable to access SEARCH_DIRECTORY_PLAN_NAME");
    }
    return SearchDirectoryWindowHandler.searchDirectoryWindowPlan;
  }



	/* Instance Variables */
  protected Button            caseSensitiveButton,
		              searchSubdirectoriesButton,
		              searchDirectoryButton,
		              stopSearchButton;
  protected TextField         searchDirectoryTextfield,
			      searchStringTextfield,
			      searchingTextfield;
  protected ScrollGroup       foundScrollgroup;
  protected ExtendedTextView  foundTextview;

  protected StringBuffer      foundStringBuffer;
  protected boolean           isSearching, isStopping;
  protected Boolean           synchronizeLock;

  protected TaskBrowserWindowHandler  taskBrowserWindowHandler;


	/* Constructors */
  public SearchDirectoryWindowHandler ( TaskBrowserWindowHandler
					          theTaskBrowserWindowHandler )
  {
    createAndInitialize ( false, theTaskBrowserWindowHandler  );
  }

  public SearchDirectoryWindowHandler ( boolean   theUseExternalWindow,
					TaskBrowserWindowHandler
					          theTaskBrowserWindowHandler )
  {
    createAndInitialize ( theUseExternalWindow, theTaskBrowserWindowHandler );
  }

	/* Constructors can't be synchronized. */
	/* And this needs to be synchronized since we use the static plans */
  private synchronized
     boolean createAndInitialize ( boolean        theUseExternalWindow,
				   TaskBrowserWindowHandler
					          theTaskBrowserWindowHandler )
  {
	/* Initialize variables */
    foundStringBuffer = new StringBuffer();
    isSearching       = false;
    isStopping        = false;
    synchronizeLock   = new Boolean ( true );
    taskBrowserWindowHandler = theTaskBrowserWindowHandler;

	/* Let the super-class do all the work... */
    if ( super . createAndInitialize ( 
		   SearchDirectoryWindowHandler.getSearchDirectoryWindowPlan(),
		   theUseExternalWindow,
		   SearchDirectoryWindowHandler.TITLE,
		   "SearchDirectoryWindowHandler",
		   theTaskBrowserWindowHandler . getVDT() )
	 == false )
    {
      return false;
    }

	/* Lets find our instance variables */
    caseSensitiveButton
      = (Button)
         SearchDirectoryWindowHandler.getSearchDirectoryWindowPlan()
          . componentNamed (
		     SearchDirectoryWindowHandler.CASE_SENSITIVE_BUTTON_NAME );
    searchSubdirectoriesButton
      = (Button)
         SearchDirectoryWindowHandler.getSearchDirectoryWindowPlan()
          . componentNamed (
		     SearchDirectoryWindowHandler.SUBDIRECTORIES_BUTTON_NAME );
    searchDirectoryButton
      = (Button)
         SearchDirectoryWindowHandler.getSearchDirectoryWindowPlan()
          . componentNamed ( SearchDirectoryWindowHandler.SEARCH_BUTTON_NAME );
    stopSearchButton
      = (Button)
         SearchDirectoryWindowHandler.getSearchDirectoryWindowPlan()
          . componentNamed (   SearchDirectoryWindowHandler.STOP_BUTTON_NAME );
    searchDirectoryTextfield
      = (TextField)
         SearchDirectoryWindowHandler.getSearchDirectoryWindowPlan()
          . componentNamed (
		       SearchDirectoryWindowHandler.DIRECTORY_TEXTFIELD_NAME );
    searchStringTextfield
      = (TextField)
         SearchDirectoryWindowHandler.getSearchDirectoryWindowPlan()
          . componentNamed (
		   SearchDirectoryWindowHandler.SEARCH_STRING_TEXTFIELD_NAME );
    searchingTextfield
      = (TextField)
         SearchDirectoryWindowHandler.getSearchDirectoryWindowPlan()
          . componentNamed (
		       SearchDirectoryWindowHandler.SEARCHING_TEXTFIELD_NAME );
    foundScrollgroup
      = (ScrollGroup)
         SearchDirectoryWindowHandler.getSearchDirectoryWindowPlan()
          . componentNamed (
			SearchDirectoryWindowHandler. FOUND_SCROLLGROUP_NAME );

	/* Error-check */
    if (   ( caseSensitiveButton        == null )
	|| ( searchSubdirectoriesButton == null )
	|| ( searchDirectoryButton      == null )
	|| ( stopSearchButton           == null )
	|| ( searchDirectoryTextfield   == null )
	|| ( searchStringTextfield      == null )
	|| ( searchingTextfield         == null )
	|| ( foundScrollgroup           == null ) )
    {
      System.err.println ( "[NewTaskWindowHandler:createAndInitialize]  "
        + "Warning:  One or more Plan objects is NULL!!!\n"
        + "    caseSensitiveButton........: " + caseSensitiveButton      + "\n"
        + "    searchSubdirectoriesButton.: " + searchSubdirectoriesButton+"\n"
	+ "    searchDirectoryButton......: " + searchDirectoryButton    + "\n"
	+ "    stopSearchButton...........: " + stopSearchButton         + "\n"
        + "    searchDirectoryTextfield...: " + searchDirectoryTextfield + "\n"
	+ "    searchStringTextfield......: " + searchStringTextfield    + "\n"
	+ "    searchingTextfield.........: " + searchingTextfield       + "\n"
	+ "    foundScrollgroup...........: " + foundScrollgroup         + "\n"
	+ "------------------------------------------------------------" );
    }


	/* Reset the plan so that we can use it again later on... */
    SearchDirectoryWindowHandler.getSearchDirectoryWindowPlan()
      . releaseObjects();


	/* Perform object reconfigurations... */
    DefaultTextFilter.setDefaultTextFilter ( searchDirectoryTextfield );
    DefaultTextFilter.setDefaultTextFilter ( searchStringTextfield    );
    DefaultTextFilter.setDefaultTextFilter ( searchingTextfield       );
    searchDirectoryTextfield . setOwner ( this );
    searchingTextfield       . setOwner ( this );
    foundTextview = new ExtendedTextView ( 0, 0,
			  foundScrollgroup . scrollView() . bounds() . width,
			  foundScrollgroup . scrollView() . bounds() . height);
    foundScrollgroup . setContentView ( foundTextview );
    foundScrollgroup . setBackgroundColor( foundTextview . backgroundColor() );
    foundTextview . setOwner ( this );
    foundTextview . setEditable ( false );
    foundTextview . setFont ( Font.fontNamed ( "Helvetica", Font.PLAIN, 18 ) );
    foundTextview . setHorizResizeInstruction( View.WIDTH_CAN_CHANGE  );
    foundTextview . setVertResizeInstruction ( View.HEIGHT_CAN_CHANGE );
    setTextFieldText ( getSearchDirectoryTextfield(),
		       System.getProperty ( "user.dir", "" ) );


	/* Report that we succeeded. */
    return true;
  }


  public RootView  getRootView()   { return getStopSearchButton().rootView(); }

  public Button    getCaseSensitiveButton()   { return caseSensitiveButton;   }
  public Button    getSearchSubdirectoriesButton()
				         { return searchSubdirectoriesButton; }
  public Button    getSearchDirectoryButton() { return searchDirectoryButton; }
  public Button    getStopSearchButton()      { return stopSearchButton;      }

  public TextField  getSearchDirectoryTextfield()
					   { return searchDirectoryTextfield; }
  public TextField  getSearchStringTextfield(){ return searchStringTextfield; }
  public TextField  getSearchingTextfield()   { return searchingTextfield;    }

  public ExtendedTextView  getFoundTextview() { return foundTextview;         }
  public ScrollGroup  getFoundScrollgroup()   { return foundScrollgroup;      }
  public StringBuffer getFoundStringBuffer()  { return foundStringBuffer;     }

  public TaskBrowserWindowHandler getTaskBrowserWindowHandler()
					   { return taskBrowserWindowHandler; }

  public boolean getIsSearching() { return isSearching; }

  public void    setIsSearching ( boolean theIsSearching )
  {
    if ( theIsSearching )
    {
      getCaseSensitiveButton()        . setEnabled  ( false );
      getSearchSubdirectoriesButton() . setEnabled  ( false );
      getSearchDirectoryButton()      . setEnabled  ( false );
      getStopSearchButton()           . setEnabled  ( true  );
      getSearchDirectoryTextfield()   . setEditable ( false );
      getSearchStringTextfield()      . setEditable ( false );
      getFoundTextview()     . setString ( "" );
      getFoundStringBuffer() . setLength ( 0 );
    }
    else
    {
      getCaseSensitiveButton()        . setEnabled  ( true  );
      getSearchSubdirectoriesButton() . setEnabled  ( true  );
      getSearchDirectoryButton()      . setEnabled  ( true  );
      getStopSearchButton()           . setEnabled  ( false );
      getSearchDirectoryTextfield()   . setEditable ( true  );
      getSearchStringTextfield()      . setEditable ( true  );
      setIsStopping ( false );
    }
    isSearching = theIsSearching;
  }

  public boolean getIsStopping()
  {
    synchronized ( synchronizeLock )
    {
      return isStopping;
    }
  }

  public void setIsStopping ( boolean theIsStopping )
  {
    if ( theIsStopping   &&   ( getIsSearching() == false ) )
    {
      System.err.println ( "[SearchDirectoryWindowHandler:setIsStopping]  "
		   + "Error:  setIsStopping ( true ) while not searching." );
      return;
    }

    synchronized ( synchronizeLock )
    {
      isStopping = theIsStopping;
    }
  }


  public boolean setSearchDirectory ( String  theSearchDirectory )
  {
    if (   ( theSearchDirectory            != null        )
        && ( theSearchDirectory . length()  > 0           )
	&& ( getSearchDirectoryTextfield() . isEditable() ) )
    {
      setTextFieldText ( getSearchDirectoryTextfield(), theSearchDirectory );
      return true;
    }
    else
      return false;
  }


	/* ExtendedTarget Interface */
  public boolean canPerformCommand ( String theCommand ) 
  {
    return
         theCommand
           . equals ( SearchDirectoryWindowHandler.SEARCH_DIRECTORY_COMMAND )
      || theCommand
           . equals ( SearchDirectoryWindowHandler.STOP_SEARCH_COMMAND    );
  }


	/* Target Interface */
  public void performCommand ( String theCommand, Object theData )
  {

    if ( theCommand . equals ( SearchDirectoryWindowHandler
				     . SEARCH_DIRECTORY_COMMAND  ) )
    {
      beginSearch();
    }

    else if ( theCommand . equals ( SearchDirectoryWindowHandler
				     . STOP_SEARCH_COMMAND     ) )
    {
      setIsStopping ( true );
    }



    else if ( theCommand
	        . equals ( SearchDirectoryWindowHandler . SEARCHING_FILE ) )
    {
      if ( theData instanceof String )
      {
	setTextFieldText ( getSearchingTextfield(), (String) theData );
      }
      else
      {
	System.err.println (
	    "[SearchDirectoryWindowHandler:performCommand(SEARCHING_FILE)]  "
	    + "Warning:  theData is not a String." );
      }
    }



    else if ( theCommand 
	        . equals ( SearchDirectoryWindowHandler . FOUND_FILE ) )
    {
      if ( theData instanceof String )
      {
	getFoundTextview() . importHTMLFromString ( ( (String) theData )
						    + "\n</BODY>\n</HTML>\n" );
      }
      else
      {
	System.err.println (
	    "[SearchDirectoryWindowHandler:performCommand(FOUND_FILE)]  "
	    + "Warning:  theData is not a String." );
      }
    }


    else if ( theCommand
	        . equals ( SearchDirectoryWindowHandler . FINISHED_SEARCHING ))
    {
      if ( theData instanceof StringBuffer )
      {
	setTextFieldText ( getSearchingTextfield(), "" );
	foundStringBuffer = (StringBuffer) theData;
	getFoundTextview() . importHTMLFromString ( ((StringBuffer) theData)
						     . toString() );
	setIsSearching ( false );
      }
      else
      {
	System.err.println (
	    "[SearchDirectoryWindowHandler:performCommand(FINISHED_SEARCHING)]"
	    + "  Warning:  theData is not a StringBuffer." );
      }
    }


    else
    {
      super . performCommand ( theCommand, theData );
    }

  } /* public void performCommand ( String theCommand, Object theData ) */


	/* TextFieldOwner Interface */
  public void    textWasModified     ( TextField theTextField ) { }
  public void    textEditingDidBegin ( TextField theTextField ) { }
  public boolean textEditingWillEnd  ( TextField theTextField,
				       int       theEndCondition,
				       boolean   theContentsChanged )
							       { return true; }

  public void    textEditingDidEnd   ( TextField theTextField,
				       int       theEndCondition,
				       boolean   theContentsChanged )
  {
	/* Ignore any pending events after, or while, our window goes away. */
    if ( getWindow() . isVisible() == false )
      return;

    if ( theTextField == getSearchDirectoryTextfield() )
    {
    }
  }


	/* TextViewOwner Interface */
  public  void attributesDidChange(TextView t, Range r){}
  public  void attributesWillChange(TextView t, Range r){}
  public  void textEditingDidBegin(TextView t) {}
  public  void textEditingDidEnd(TextView t) {}
  public  void selectionDidChange ( TextView theTextView )  {}
  public  void textWillChange ( TextView theTextView, Range theRange )  {}
  public  void textDidChange ( TextView theTextView, Range theRange )  {}

  public  void linkWasSelected ( TextView theTextView,
 				 Range    theRange,
				 String   theString )
  {
    getTaskBrowserWindowHandler() . addURL ( theString );
  }


	/********************************************/
	/************   Searching Code   ************/
	/********************************************/
  protected void beginSearch()
  {
    if ( getIsSearching() )
    {
      System.err.println ( "[SearchDirectoryWindowHandler:beginSearch]  "
			   + "Warning:  Already Searching..." );
      return;
    }

    if (   ( getSearchDirectoryTextfield() . stringValue()            == null )
	|| ( getSearchDirectoryTextfield() . stringValue() . length() <= 0    )
	)
    {
      runAlert ( "Error:",
		 "No Search Directory has been specified",
		 "OK", null, null );
      return;
    }

    if (   ( getSearchStringTextfield() . stringValue()            == null )
	|| ( getSearchStringTextfield() . stringValue() . length() <= 0    ) )
    {
      runAlert ( "Error:",
		 "No Search String has been specified",
		 "OK", null, null );
      return;
    }

    if ( ( new File ( getSearchDirectoryTextfield() . stringValue() ) )
	   . exists() == false )
    {
      runAlert ( "Error:",
		 "The Search Directory does not exist!",
		 "OK", null, null );
      return;
    }


    setIsSearching ( true );

    Thread  thread = new Thread ( this );
    thread . setPriority ( Thread.MIN_PRIORITY );
    thread . start();
  }


	/************************************************/
	/************   Runnable Interface   ************/
	/************************************************/
  public void run()
  {
    String   initialDirectory = getSearchDirectoryTextfield() . stringValue();
    String   searchString     = getSearchStringTextfield()    . stringValue();
    boolean  ignoreCase       = getCaseSensitiveButton() . state() == false;
    boolean  searchSubdirectories = getSearchSubdirectoriesButton() . state();

    int           i;
    StringBuffer  foundText = new StringBuffer();

	/* Strip multiple whitespace from searchString */
	/* Reuse the foundText object for this...      */
    for ( i=0;  (searchString != null) && (i < searchString . length());  i++ )
    {
      if ( Character.isWhitespace ( searchString . charAt ( i ) ) )
      {
	if (   ( i == 0 )
	    || ( Character.isWhitespace ( foundText . charAt (
						 foundText . length() - 1 ) )
		 == false ) )
	{
	  foundText . append ( " " );
	}
      }
      else
      {
	   /* Should we be case-insensitive? */
	if ( ignoreCase )
	  foundText . append ( Character.toUpperCase ( 
					       searchString . charAt ( i ) ) );
	else
	  foundText . append ( searchString . charAt ( i ) );
      }
    }
    searchString = foundText.toString();


	/* Erase the foundText object... */
    foundText . setLength ( 0 );


	/* Start our HTML text... */
    foundText . append ( "<HTML>\n<HEAD>\n<TITLE>Directory Search of " );
    foundText . append ( initialDirectory );
    foundText . append ( " for \"" );
    foundText . append ( searchString );
    foundText . append ( "\"</TITLE>\n</HEAD>\n<BODY>\n\n" );

	/* Append the text so far to the foundTextview */
    Application . application()
      . performCommandLater ( this,
			      SearchDirectoryWindowHandler.FOUND_FILE,
			      foundText.toString(), false );

	/* Idiot-check */
    if (   ( initialDirectory            != null )
	&& ( initialDirectory . length()  > 0    )
	&& ( searchString                != null )
	&& ( searchString . length()      > 0    ) )
      
    {
	/* Start the search */
      checkDirectoryOrFile ( initialDirectory, searchString,
			     ignoreCase, searchSubdirectories,
			     foundText );
    }

	/* Finish our HTML text... */
    foundText . append ( "\n</BODY>\n</HTML>\n" );


	/* Terminate the search */
    Application . application()
      . performCommandLater ( this,
			      SearchDirectoryWindowHandler.FINISHED_SEARCHING,
			      foundText, false );
  }


  public void checkDirectoryOrFile ( String        theDirectoryOrFile,
				     String        theSearchString,
				     boolean       theIgnoreCase,
				     boolean       theSearchSubdirectories,
				     StringBuffer  theFoundText )
  {
	/* Idiocy checks */
    if (   ( theDirectoryOrFile            == null )
	|| ( theDirectoryOrFile . length() <= 0    )
	|| ( theSearchString               == null )
	|| ( theSearchString . length()    <= 0    )
	|| ( getIsStopping()                       ) )
      return;

	/* Let the use know what we are doing... */
    Application . application()
      . performCommandLater ( this,
			      SearchDirectoryWindowHandler.SEARCHING_FILE,
			      theDirectoryOrFile, false );

	/* The point to using threads is NOT to hog the CPU... */
    Thread.currentThread() . yield();



    int   i;
    File  file = new File ( theDirectoryOrFile );

    if ( file . exists() == false )
    {
      System.err.println (
	  "[SearchDirectoryWindowHandler:checkDirectoryOrFile]  Warning:  "
	  + "File does not exist: " + theDirectoryOrFile );
      return;
    }


    else if (   ( file . isDirectory()    )
	     && ( theSearchSubdirectories ) )
    {
      String  directoryList[] = file . list();

	/* Lets do things in alphabetical order... */
      Sort.sortStrings ( directoryList, 0, directoryList.length, true, false );

      for ( i=0;  i < directoryList.length;  i++ )
      {
	checkDirectoryOrFile ( file . getAbsolutePath()
			       + System.getProperty ( "file.separator", "/" )
			       + directoryList [ i ],
			       theSearchString,
			       theIgnoreCase, theSearchSubdirectories,
			       theFoundText );

	    /* If we have decided to terminate... */
	if ( getIsStopping() )
	  return;
      }
    } /* else if ( ( file . isDirectory() ) && ( theSearchSubdirectories ) ) */


    else if ( file . isFile() )
    {
      Reader  reader;
      int     foundTextLenght = theFoundText . length();

      try
      {
	reader = new BufferedReader ( new FileReader ( file ) );
      }
      catch ( FileNotFoundException  theException )
      {
	reader = null;
	System.err.println ( 
	    "[SearchDirectoryWindowHandler:checkDirectoryOrFile]  Warning:  "
	    + "File not Found:  \"" + theDirectoryOrFile
	    + "\"    Exception:\"" + theException + "\"" );
      }

      if ( reader != null )
      {
	if ( searchReader ( reader, theSearchString, theIgnoreCase ) >= 0 )
	{
	  theFoundText . append ( "<A HREF=file:"    );
	  theFoundText . append ( theDirectoryOrFile );
	  theFoundText . append ( ">"                );
	  theFoundText . append ( theDirectoryOrFile );
	  theFoundText . append ( "</A>\n"           );

	  Application . application()
	    . performCommandLater ( this,
				    SearchDirectoryWindowHandler.FOUND_FILE,
				    theFoundText.toString(), false );
	}
      }
    } /* else if ( file . isFile() ) */
    
  }



  static int[] makeTable ( String theMatchString )
  {
    int    i, j;
    int[]  table = new int [ theMatchString . length() ];

    table [ 0 ] = -1;
    table [ 1 ] = 0;
    for ( i=2;  i < theMatchString.length();  i++ )
    {
      j = table [ i - 1 ] + 1;

      while (   ( j > 0 )
	     && (    theMatchString . charAt ( i - 1)
		  != theMatchString . charAt ( j - 1) ) )
      {
	j = table [ j - 1 ] + 1;
      }
      table [ i ] = j;
    }

    return table;
  }

  static int getNextChar ( Reader   theReader,
			   int      thePreviousChar,
			   boolean  theIgnoreCase   )
  {
    int   readChar;

    if (   (  thePreviousChar >= 0                           )
	&& ( Character.isWhitespace ( (char) thePreviousChar ) ) )
    {
      thePreviousChar = ' ';
    }

    do
    {
      try
      {
	readChar = theReader . read();
      }
      catch ( IOException  theException )
      {
	System.err.println (
	    "[SearchDirectoryWindowHandler:getNextChar]  Warning:  "
	    + "IOExcetion Encountered:  " + theException );
	readChar = -1;
      }

      if ( readChar >= 0 )
      {
	if ( Character.isWhitespace ( (char) readChar ) )
	{
	  readChar = ' ';
	}

	else if ( theIgnoreCase )
	  readChar = Character.toUpperCase ( (char) readChar );
      }

    } while (   ( thePreviousChar == ' ' )
	     && ( readChar        == ' ' ) );

    return readChar;
  }


  int searchReader ( Reader   theReader,
		     String   theMatchString,
		     boolean  theIgnoreCase  )
  {
    int  table[] = makeTable ( theMatchString );
    int  length = 0;
    int  m = 0;
    int  previousChar = -1;
    int  currentChar  = getNextChar ( theReader, previousChar, theIgnoreCase );

    while ( currentChar != -1 )
    {
	/* Every N bytes... */
      if (   ( length > 0 )   &&   ( (length % 100000) == 0 )   )
      {
	   /* Check to see if we are stopping... */
	if ( getIsStopping() )
	  return -1;

	   /* The point to using threads is NOT to hog the CPU... */
	Thread.currentThread() . yield();
      }

      if ( currentChar == theMatchString . charAt ( m ) )
      {
	previousChar = currentChar;
	currentChar  = getNextChar ( theReader, previousChar, theIgnoreCase );
	length++;
	m++;
      }
      else
      {
	m = table [ m ];
	if ( m == -1 )
	{
	  previousChar = currentChar;
	  currentChar  = getNextChar( theReader, previousChar, theIgnoreCase );
	  length++;
	  m++;
	}
      }
      if ( m == theMatchString.length() )
	return length - m;
    }

    return -1;
  }

  static int index ( String theMatchString, String theSourceString )
  {
    int  table[] = makeTable ( theMatchString );
    int  s = 0, m = 0;

    while ( s < theSourceString.length() )
    {
      if ( theSourceString . charAt ( s ) == theMatchString . charAt ( m ) )
      {
	s++;
	m++;
      }
      else
      {
	m = table [ m ];
	if ( m == -1 )
	{
	  m++;
	  s++;
	}
      }
      if ( m == theMatchString.length() )
	return s - m;
    }

    return -1;
  }

}

