
#include "_TDL_SpawnStatementData.H"
#include "_TDL_SpawnStatementTreeNode.H"


/*virtual*/
_TDL_SpawnStatementData::~_TDL_SpawnStatementData()
{
  if ( isNotStarted() == TRUE )
  {
    destroy();
  }

	/* Sometimes a object is used after it is deleted... */
	/* It's nice to be able to pick up on this... */
  state = _TDL_SpawnStatementData::DELETED;
}


_TDL_SpawnStatementTreeNode *
_TDL_SpawnStatementData::createAndAttachNewSpawnStatementTreeNode()
{
  _TDL_SpawnStatementTreeNode * newSpawnStatementTreeNode
    = new _TDL_SpawnStatementTreeNode ( * this );

  if ( getSpawnStatementTreeNodeSlistNonConst()
         . appendNode ( newSpawnStatementTreeNode ) == FAILURE )
  {
    TDL::getLogStream()
      << "[_TDL_SpawnStatementData:createAndAttachNewSpawnStatementTreeNode]  "
      << "Error:  appendNode FAILED.   Returning NULL!!!" << endl;

    delete newSpawnStatementTreeNode;
    return (_TDL_SpawnStatementTreeNode *) NULL;
  }

  return newSpawnStatementTreeNode;
}



status_t
_TDL_SpawnStatementData::removeSpawnStatementTreeNode (
	      _TDL_SpawnStatementTreeNode * theSpawnStatementTreeNodeToRemove )
{
  status_t returnValue
	     = (    getSpawnStatementTreeNodeSlistNonConst()
		      . removeNode ( theSpawnStatementTreeNodeToRemove )
		 == theSpawnStatementTreeNodeToRemove ) ? SUCCESS : FAILURE;

  if ( returnValue == FAILURE )
  {
    TDL::getLogStream()
      << "[_TDL_SpawnStatementData:removeSpawnStatementTreeNode]  "
      << "Error:  removeNode ( " << ((void*)theSpawnStatementTreeNodeToRemove)
      << " ) FAILED." << endl;
  }

  return returnValue;
}



const char *
_TDL_SpawnStatementData::getStateString() const
{
  switch ( getState() )
  {
    case _TDL_SpawnStatementData::NOT_ALLOCATED:  return "NOT_ALLOCATED";
    case _TDL_SpawnStatementData::ALLOCATED:      return "ALLOCATED";
    case _TDL_SpawnStatementData::RUNNING:        return "RUNNING";
    case _TDL_SpawnStatementData::DESTROYED:      return "DESTROYED";
    case _TDL_SpawnStatementData::DELETED:        return "***DELETED***";
    default:                                      return "***UNKNOWN***";
  }
}



status_t
_TDL_SpawnStatementData::startRunning ( )
{
  switch ( getState() )
  {
    case _TDL_SpawnStatementData::NOT_ALLOCATED:
      if ( allocate() == FAILURE )
      {
	TDL::getLogStream()
	  << "[_TDL_SpawnStatementData::startRunning(\"" << getName()
	  << "\")]  ERROR:  allocate() FAILED!!!  Aborting..."
	  << endl;
	return FAILURE;
      }

      /* NO BREAK OR RETURN!!!  GO DIRECTLY TO "ALLOCATED" case */

    case _TDL_SpawnStatementData::ALLOCATED:
	/* Record that we are now "Running" this node. */
      state = _TDL_SpawnStatementData::RUNNING;
      return SUCCESS;


    case _TDL_SpawnStatementData::RUNNING:
	/* It's already Running.  What are we going to do?  Fail? */
      return SUCCESS;


    case _TDL_SpawnStatementData::DESTROYED:
      TDL::getLogStream()
	<< "[_TDL_SpawnStatementData::startRunning(\"" << getName()
	<< "\")]  ERROR:  Node has already been destroyed!" << endl;
      return FAILURE;

    case _TDL_SpawnStatementData::DELETED:
      TDL::getLogStream()
	<< "[_TDL_SpawnStatementData::startRunning(\"" << getName()
	<< "\")]  ERROR:  Object has already been DELETED!" << endl;
      return FAILURE;

    default:
      TDL::getLogStream()
	<< "[_TDL_SpawnStatementData::startRunning(\"" << getName()
	<< "\")]  ERROR:  Unknown state: " << getState() << endl;
      return FAILURE;
  }
}



status_t
_TDL_SpawnStatementData::allocate ()
{
  switch ( getState() )
  {
    case _TDL_SpawnStatementData::NOT_ALLOCATED:

	/* Lets double check this... */
      if (    getSpawnStatement() . getAllocationFunction()
	   == _TDL_AllocationFunctionType ( NULL ) )
      {
	TDL::getLogStream()
	  << "[_TDL_SpawnStatementData::allocate(\"" << getName()
	  << "\")]  ERROR:  getAllocationFunction() is NULL!!!  Aborting..."
	  << endl;
	return FAILURE;
      }

	/* Default-Allocate that puppy... */
      setTCMTaskTreeDirectly ( ( * (getSpawnStatement()
				      . getAllocationFunction()) )
			       ((const char *) NULL) );

	/* It's not paranoia if the compiler/user is really out to get you...*/
      if ( getNodeHandle() . operator*() == NULL )
      {
	TDL::getLogStream()
	  << "[_TDL_SpawnStatementData::allocate(\"" << getName() << "\")]  "
	  << "Warning:  getAllocationFunction() seems to have returned NULL!"
	  << endl;
	return FAILURE;
      }
      state = _TDL_SpawnStatementData::ALLOCATED;
      return SUCCESS;


    case _TDL_SpawnStatementData::ALLOCATED:
    case _TDL_SpawnStatementData::RUNNING:
	/* It's already allocated.  What are we going to do?  Fail? */
      return SUCCESS;


    case _TDL_SpawnStatementData::DESTROYED:
      TDL::getLogStream()
	<< "[_TDL_SpawnStatementData::allocate(\"" << getName()
	<< "\")]  Warning:  Node has already been destroyed!" << endl;
      return FAILURE;

    case _TDL_SpawnStatementData::DELETED:
      TDL::getLogStream()
	<< "[_TDL_SpawnStatementData::allocate(\"" << getName()
	<< "\")]  ERROR:  Object has already been DELETED!" << endl;
      return FAILURE;

    default:
      TDL::getLogStream()
	<< "[_TDL_SpawnStatementData::allocate(\"" << getName()
	<< "\")]  ERROR:  Unknown state: " << getState() << endl;
      return FAILURE;
  }
}



status_t
_TDL_SpawnStatementData::destroy ( BOOLEAN theCanStillBeReferenced /*=FALSE*/ )
{
	/* Might as well note that we have been destroyed. */
  for ( _TDL_Snode * node  = getSpawnStatementTreeNodeSlist() . getFirstNode();
		     node != (_TDL_Snode *) NULL;
		     node  = node -> _TDL_Snode::getNextNode() )
  {
    ((_TDL_SpawnStatementTreeNode *) node) -> setIsDestroyedSubtree();
  }


  switch ( getState() )
  {
    case _TDL_SpawnStatementData::NOT_ALLOCATED:
	/* If we are not using this node anymore
         *    Or (if we need to keep the allocated node handle around
	 *        AND we can't allocate it... )
	 */
      if (   (     theCanStillBeReferenced == FALSE     )
	  || (   ( theCanStillBeReferenced == TRUE    )
	      && ( allocate()              == FAILURE ) ) )
      {
		/* If the allocate() returned FAILURE */
	if ( theCanStillBeReferenced == TRUE )
	{
	  TDL::getLogStream()
	    << "[_TDL_SpawnStatementData::destroy(\"" << getName() << "\")]  "
	    << "WARNING:  Unable to allocate() node before destroy()'ing "
	    << "it.  Using destroyed node of NULL!" << endl;
	}
	state = _TDL_SpawnStatementData::DESTROYED;
	getNodeHandleNonConst() . operator= ( (Task_Tree_Node *) NULL );
	return SUCCESS;
      }

      /* NO BREAK STATEMENT.  Continue on with "ALLOCATED" case. */

    case _TDL_SpawnStatementData::ALLOCATED:
      if ( TCM_DeallocateNode ( getNodeHandleNonConst() ) == TCM_Ok )
      {
	state = _TDL_SpawnStatementData::DESTROYED;
	if ( theCanStillBeReferenced == FALSE )
	  getNodeHandleNonConst() . operator= ( (Task_Tree_Node *) NULL );
	return SUCCESS;
      }
      else
      {
	TDL::getLogStream()
	  << "[_TDL_SpawnStatementData::destroy(\"" << getName()
	  << "\")]  FAILURE:  TCM_DeallocateNode did *NOT* return TCM_Ok.  "
	  << "\"destroy()\" operation aborted!" << endl;
	return FAILURE;
      }


    case _TDL_SpawnStatementData::RUNNING:
      TDL::getLogStream()
	<< "[_TDL_SpawnStatementData::destroy(\"" << getName()
	<< "\")]  ERROR:  Object is currently running.  "
	<< "Destruction must occur *BEFORE* it starts running!"
	<< "\"destroy()\" operation aborted!" << endl;
      return FAILURE;


    case _TDL_SpawnStatementData::DESTROYED:
	/* It's already Destroyed.  What are we going to do?  Fail? */
      return SUCCESS;


    case _TDL_SpawnStatementData::DELETED:
      TDL::getLogStream()
	<< "[_TDL_SpawnStatementData::destroy(\"" << getName()
	<< "\")]  ERROR:  Object has already been DELETED!" << endl;
      return FAILURE;

    default:
      TDL::getLogStream()
	<< "[_TDL_SpawnStatementData::destroy(\"" << getName()
	<< "\")]  ERROR:  Unknown state: " << getState() << endl;
      return FAILURE;
  }
}



status_t
_TDL_SpawnStatementData::destroyIfUnused ( BOOLEAN theCanStillBeReferenced  )
{								  /* = FALSE */
	/* Might as well note that we have been destroyed. */
  for ( _TDL_Snode * node  = getSpawnStatementTreeNodeSlist() . getFirstNode();
		     node != (_TDL_Snode *) NULL;
		     node  = node -> _TDL_Snode::getNextNode() )
  {
    ((_TDL_SpawnStatementTreeNode *) node) -> setIsDestroyedSubtree();
  }

  if ( isStarted() == TRUE )
    return SUCCESS;
  else
    return destroy ( theCanStillBeReferenced );
}


status_t
_TDL_SpawnStatementData::doDestroy ( BOOLEAN   theCanStillBeReferenced,
				     BOOLEAN   theDestroyIfUnused      )
{
  if ( theDestroyIfUnused == TRUE )
    return destroyIfUnused ( theCanStillBeReferenced );
  else
    return destroy         ( theCanStillBeReferenced );
}



status_t
_TDL_SpawnStatementData::startRunningNullSpawnStatementData()
{
  if ( getState() == _TDL_SpawnStatementData::RUNNING )
    return SUCCESS;

  if (   (   ( getState()   == _TDL_SpawnStatementData::NOT_ALLOCATED )
	  || ( getState()   == _TDL_SpawnStatementData::ALLOCATED     ) )

      && (    getSpawnStatement() . getAllocationFunction()
	   == _TDL_AllocationFunctionType ( NULL )                      ) )
  {
    getNodeHandleNonConst() = NULL;
    state = _TDL_SpawnStatementData::RUNNING;
    return SUCCESS;
  }
  else
  {
    return FAILURE;
  }
}



	/* Allow automatic conversion to TCM_Task_Tree_Ref.
	 * (Will also transparently allocate() unallocated instances,
	 *  performing lazy allocation.)
	 */
_TDL_SpawnStatementData::operator const TCM_Task_Tree_Ref & ()
{
  if ( getState() == _TDL_SpawnStatementData::NOT_ALLOCATED )
  {
    if ( allocate() == FAILURE )
    {
      TDL::getLogStream()
	<< "[_TDL_SpawnStatementData::operator const TCM_Task_Tree_Ref & (\""
	<< getName() << "\")]  ERROR:  allocate() FAILED!!!" << endl;
    }
  }
  return getNodeHandle();
}


	/* iostreamBase interface. */
/*virtual*/ ostream &
_TDL_SpawnStatementData::printObject (
				   ostream    & theOstream,
				   const char * theIndentString /*=""*/ ) const
{
  char  subIndentString [ iostreamBase::MAXIMUM_INDENT ];
  iostreamBase::createSubIndentString ( subIndentString, theIndentString );

  theOstream << theIndentString << "-Begin- _TDL_SpawnStatementData  "
	     << ((void *)this)  << endl

	     << theIndentString
	     << " Associated with: " << ((void *) & (getSpawnStatement()) )
	     << "  ( \"" << getName() << "\" )" << endl

	     << theIndentString << " State..........: \""
	     << getStateString() << "\"  (" << getState() << ")" << endl

	     << theIndentString << " handleRef......: "
	     << ((void *)  ( getNodeHandle() . operator*() )) << endl

	     << theIndentString << " SpawnStatementTreeNodes:" << endl;

	/* Do this manually here to avoid an endless loop. */
  for ( _TDL_Snode * node  = getSpawnStatementTreeNodeSlist() . getFirstNode();
	             node != (_TDL_Snode *) NULL;
	             node  = node -> getNextNode()
       )
  {
    theOstream << theIndentString << "   _TDL_SpawnStatementTreeNode:  "
	       << ((void *) node)  << endl;
  }

  theOstream << theIndentString << "--End-- _TDL_SpawnStatementData  "
	     << ((void *)this)  << endl;

  return theOstream;
}



	/* These operations are disabled (prohibited) & unimplemented */
/*private*/
_TDL_SpawnStatementData::_TDL_SpawnStatementData (
			      const _TDL_SpawnStatementData & theObjectToCopy )
    :  _TDL_Snode                  (                                        ),
       state                       ( _TDL_SpawnStatementData::NOT_ALLOCATED ),
       tcmTaskTreeRefHandle        (                                        ),
       spawnStatement              ( theObjectToCopy . getSpawnStatement()  ),
       spawnStatementTreeNodeSlist ( TRUE /* Delete contained objects */    )
{
  _TDL_MARKUSED ( theObjectToCopy );
  TDL::getLogStream()
    << "_TDL_SpawnStatementData copy constructor bug." << endl;
}

/*private*/
_TDL_SpawnStatementData &
_TDL_SpawnStatementData::operator = (
			      const _TDL_SpawnStatementData & theObjectToCopy )
{
  _TDL_MARKUSED ( theObjectToCopy );
  TDL::getLogStream()
    << "_TDL_SpawnStatementData operator = bug." << endl;
  return *this;
}
